/* Runaway Robot

 Play tag with your robot! With an ultrasonic 
 distance sensor, it's capable of detecting and avoiding
 obstacles, never (???) bumping into walls again!
  
 You'll need to attach an untrasonic range finder to TK1.
 
 Circuit:
 * Arduino Robot
 * US range finder like Maxbotix EZ10, with analog output
 
 created 1 May 2013
 by X. Yang
 modified 12 May 2013
 by D. Cuartielles
 
(Mod. 7-3-13 by B.F. This program is re-written to make use of a PING))) AND the upper pot.)
(This program uses a reversing routine that helps A LOT in preventing the bot from "getting stuck".)
(Awesome little bot! This is JUST the beginning...)
 
 This example is in the public domain
*/
 
// include the robot library
#include <ArduinoRobot.h>

const int pingPin = TKD3; // pin is used by the sensor
const int dangPin = TKD4; // red warning light

int ctrl; // value from on-board pot
int cm; // distance in cm
int dist; // ctrl minus cm
int trn = 1; // flag to allow turning off "turn()" command

int fdlt = 89; // motor1 forward speed
int fdrt = 89; // motor2 forward speed
int rvlt = -89; // motor1 reverse speed
int rvrt = -89; // motor2 reverse speed

void setup() {
  // initialize the Robot, SD card, and display
  Serial.begin(9600);
  Robot.begin();
  Robot.beginTFT();
  Robot.beginSD();
  Robot.displayLogos();
  pinMode(TKD4, OUTPUT);
  
  // draw a face on the LCD screen
  setFace(true);
}

void loop() {
  // If the robot is blocked, turn until free
  Robot.digitalWrite(TKD4, LOW); // turn off warn LED
  ctrl = Robot.knobRead(); // read the upper pot to set the detection distance
  ctrl = map(ctrl, 0, 1023, 0, 50); // map pot to range of 0 to 50
  while(ping() < ctrl) { // This works well with the PING)))
    dist = ctrl - ping(); // distance for display purposes
    Robot.digitalWrite(TKD4, HIGH); // turn ON warn LED
    setFace(false); //shows an unhappy face
    Robot.motorsStop(); // stop the motors
    Robot.motorsWrite(rvlt, rvrt); // reverse for a bit
    delay(750); // give some time to reverse 
    Robot.motorsStop(); // stop again
    delay(1000); // wait for a moment
    if (trn == 1) {Robot.turn(90);} // can stop turning for debug
    Robot.digitalWrite(TKD4, LOW); // turn off warn LED
    setFace(true); // happy face
  }
  // if there are no objects in the way, keep moving
  Robot.digitalWrite(TKD4, LOW); // turn off warn LED
  Robot.motorsWrite(fdlt, fdrt); // THIS LINE sets the bot speed (a resonable "walking" pace)
  delay(300);
}

int ping()
{
  // establish variables for duration of the ping, 
  // and the distance result in inches and centimeters:
  long duration, cm;

  // The PING))) is triggered by a HIGH pulse of 2 or more microseconds.
  // Give a short LOW pulse beforehand to ensure a clean HIGH pulse:
  pinMode(pingPin, OUTPUT);
  Robot.digitalWrite(pingPin, LOW);
  delayMicroseconds(2);
  Robot.digitalWrite(pingPin, HIGH);
  delayMicroseconds(5);
  Robot.digitalWrite(pingPin, LOW);

  // The same pin is used to read the signal from the PING))): a HIGH
  // pulse whose duration is the time (in microseconds) from the sending
  // of the ping to the reception of its echo off of an object.
  pinMode(pingPin, INPUT);
  duration = pulseIn(pingPin, HIGH);

  // convert the time into a distance
  cm = microsecondsToCentimeters(duration);
  
  delay(100);
  return(cm);
}

long microsecondsToCentimeters(long microseconds)
{
  // The speed of sound is 340 m/s or 29 microseconds per centimeter.
  // The ping travels out and back, so to find the distance of the
  // object we take half of the distance travelled.
  return microseconds / 29 / 2;
}

// make a happy or sad face (YES! It even EMOTES!)
void setFace(boolean onOff) {
  if(onOff) {
    // if true show a happy face
    Robot.background(0, 0, 255);
    Robot.setCursor(44, 60);
    Robot.stroke(0, 255, 0);
    Robot.setTextSize(4);
    Robot.print(":)");
    Robot.text(dist,0,0); // display distance
  }else{
    // if false show an upset face
    Robot.background(255, 0, 0);
    Robot.setCursor(44, 60);
    Robot.stroke(0, 255, 0);
    Robot.setTextSize(4);
    Robot.print(":(");
    Robot.text(dist,0,0); // display distance to obstruction
  }
}


